from AC3utils import dec2hex, hex2dec
from enigma import eDVBDB
from Tools.Directories import resolveFilename, SCOPE_CONFIG, copyfile
from Screens.MessageBox import MessageBox
import codecs
import os

lamedbFile = resolveFilename(SCOPE_CONFIG) + "/lamedb"
lamedbFileBackup = lamedbFile + ".backupAC3"
lamedbFileAC3 = lamedbFile + ".AC3"

#lamedbFile = "L:\\Dreambox\\lamedb\\lamedb"
#lamedbFileNew = "L:\\Dreambox\\lamedb\\lamedbNew"

lSpecialParameters = {'c:00': 'VideoPid','c:01': 'AudioPid','c:02': 'TeletextPid','c:03': 'PCRPid','c:04': 'AC3Pid','c:07': 'DelayBitstream','c:08': 'DelayPCM'}
lSpecialParametersInvers = {}
for sParam , sValue in lSpecialParameters.iteritems():
    lSpecialParametersInvers[sValue] = sParam

class AC3Service():
    
    ChannelSID = ""
    ChannelInfo = ""
    PosInLamedb = 0
    ChannelName = ""
    NamedParameters = {}

    def setDummyService(self):
        ChannelSID = "N/A"
        ChannelInfo = "N/A"
        PosInLamedb = -1
        ChannelName = "N/A"
        NamedParameters = {}
        NamedParameters["N/A"] = "N/A"
        
class AC3lamedb():
    
    def __init__(self, session):
        self.session = session
        self.linesBeforeServices = []
        self.linesAfterServices = []
        self.serviceList = {}
        self.readServiceList()
        self.dummyService = AC3Service()
        self.dummyService.setDummyService()
        
    def readServiceList(self):
        lamedb = open( lamedbFile , "r" )
        bInServices = False
        bBeforeServices = True
        i = 0
        for line in lamedb:
            lLastPos = lamedb.tell()
            lineStrip = line.strip('\n')
            if not bInServices:
                if lineStrip == 'services':
                    self.linesBeforeServices.append(line)
                    bInServices = True
            else:
                i = i + 1
                if lineStrip == 'end':
                    bInServices = False
                    bBeforeServices = False
                else:
                    if (i % 3) == 1:
                        oCurrentService = AC3Service()
                        oCurrentService.ChannelInfo = lineStrip
                        aLineStrip = lineStrip.split(":")
                        oCurrentService.ChannelSID = aLineStrip[0]
                        oCurrentService.PosInLamedb = lLastPos
                        oCurrentService.ChannelName = ""
                        oCurrentService.NamedParameters = {}
                    elif (i % 3) == 2:
                        oCurrentService.ChannelName = lineStrip
                    elif (i % 3) == 0:
                        aParameters = []
                        aParameters = lineStrip.split(',')
                        for sParm in aParameters:
                            if sParm.__getslice__(0,1) == "p":
                                sKey = "p:"
                                sVal = sParm.__getslice__(2,sParm.__len__())
                            else:
                                sKey = sParm.__getslice__(0,4)
                                sVal = sParm.__getslice__(4,sParm.__len__())
                            if sKey in lSpecialParameters:
                                sParmName = lSpecialParameters[sKey]
                            else:
                                sParmName = sKey
                            oCurrentService.NamedParameters[sParmName] = sVal
                        self.serviceList[oCurrentService.ChannelSID] = oCurrentService
            if not bInServices:
                if bBeforeServices:
                    self.linesBeforeServices.append(line)
                else:
                    self.linesAfterServices.append(line)
        lamedb.close()
    
    def backupLameDb(self):
        if os.path.isfile(lamedbFileBackup) is True:
            self.session.openWithCallback(self.BackupYesNo, MessageBox, _("There is already a backup of the lamedb\nDo you want to overwrite it?"), MessageBox.TYPE_YESNO, -1)
        else:
            self.backupSuccess()

    def BackupYesNo(self,bAnswer):
        if bAnswer:
            self.backupSuccess()

    def backupSuccess(self):
        copyfily(lamedbFile, lamedbFileBackup)    
        sResponse = _("lamedb backup successful") 
        self.session.open(MessageBox, sResponse , MessageBox.TYPE_INFO)                        

    def restoreLameDb(self):
        if os.path.isfile(lamedbFileBackup) is not True:
            self.session.open(MessageBox,_("There is no backup of the lamedb. Action canceled"), MessageBox.TYPE_ERROR)    
        else:
            self.session.openWithCallback(self.RestoreYesNo, MessageBox, _("Do you really want to restore the lamedb?"), MessageBox.TYPE_YESNO, -1)

    def RestoreYesNo(self,bAnswer):
        if bAnswer:
            self.unlinkLameDb()
            copyfile(lamedbFileBackup, lamedbFile)    
            self.reloadLameDb()
            sResponse = _("lamedb backup successfully restored") 
            self.session.open(MessageBox, sResponse , MessageBox.TYPE_INFO)                

    def removeLameDbBackup(self):
        if os.path.isfile(lamedbFileBackup) is not True:
            self.session.open(MessageBox,_("There is no backup of the lamedb. Action canceled"), MessageBox.TYPE_ERROR)    
        else:
            self.session.openWithCallback(self.RemoveYesNo, MessageBox, _("Do you really want to remove the lamedb- backup?"), MessageBox.TYPE_YESNO, -1)            
            
    def RemoveYesNo(self,bAnswer):
        if bAnswer:
            os.system("rm %s" % (lamedbFileBackup))    
            self.reloadLameDb()
            sResponse = _("lamedb backup successfully removed") 
            self.session.open(MessageBox, sResponse , MessageBox.TYPE_INFO)                

    def saveServiceList(self):
        self.unlinkLameDb()
        lamedb = open( lamedbFile , "w" )
        #Write lines before services
        for line in self.linesBeforeServices:
            lamedb.write(line)
        #Write services
        for sInfo , oCurrentService in self.serviceList.iteritems():
            lamedb.write(oCurrentService.ChannelInfo + "\n")
            lamedb.write(oCurrentService.ChannelName + "\n")
            sParameters = ""
            for sParmName, sParm in oCurrentService.NamedParameters.iteritems():
                if sParmName in lSpecialParametersInvers:
                    sParmName = lSpecialParametersInvers[sParmName]
                if sParameters == "":
                    sParameters = sParmName + sParm
                else:
                    sParameters = sParameters + "," + sParmName + sParm
            lamedb.write(sParameters + "\n")
        #Write lines after services    
        for line in self.linesAfterServices:
            lamedb.write(line)

        lamedb.close()
        
        self.reloadLameDb()

    def unlinkLameDb(self):
        try:
            os.unlink(lamedbFile);
        except OSError:
            pass

    def reloadLameDb(self):
        print "[AC3LipSync] reloading lamedb"        
        db = eDVBDB.getInstance()
        db.reloadServicelist()

    def getServiceInfo(self,sService):
        if sService in self.serviceList:
            return self.serviceList[sService]
        else:
            return self.dummyService

    def getNamedServiceParameter(self,sService,sParameterName,bConvertToDec):
        vServiceInfo = AC3Service()
        vServiceInfo = self.getServiceInfo(sService)
        if vServiceInfo.ChannelSID != "N/A":
            if sParameterName in vServiceInfo.NamedParameters:
                if bConvertToDec:
                    return hex2dec(self.serviceList[sService].NamedParameters[sParameterName])
                else:
                    return self.serviceList[sService].NamedParameters[sParameterName]
            else:
                if bConvertToDec:
                    return 0
                else:
                    return "N/A"
        else:
            if bConvertToDec:
                return 0
            else:
                return "N/A"

    def setNamedServiceParameter(self,sService,sParameterName,iParameterValue,bConvertToHex):
        vServiceInfo = AC3Service()
        vServiceInfo = self.getServiceInfo(sService)
        if vServiceInfo.ChannelSID != "N/A":
            if sParameterName != "N/A":
                if bConvertToHex == True:
                    sParameterValue = dec2hex(iParameterValue)
                    sParameterValue = '0000' + sParameterValue
                    sParameterValue = sParameterValue.__getslice__(sParameterValue.__len__()-4,sParameterValue.__len__())
                else:
                    sParameterValue = str(iParameterValue)
                vServiceInfo.NamedParameters[sParameterName] = sParameterValue
                    
        
#x = AC3lamedb()
#x.setNamedServiceParameter('0308:00c00000:0005:0085:1:0','DelayBitstream',400)
#print x.getNamedServiceParameter('0308:00c00000:0005:0085:1:0','DelayPCM')
#x.saveServiceList()     
    