from AC3lamedb import AC3lamedb
from AC3utils import dec2hex, hex2dec
from Components.ActionMap import ActionMap, NumberActionMap
from Components.Button import Button
from Components.ConfigList import ConfigListScreen
from Components.Label import Label,MultiColorLabel
from Components.ProgressBar import ProgressBar
from Components.config import config, ConfigSubsection, ConfigSubList, ConfigInteger, ConfigFloat, getConfigListEntry
from enigma import eDVBDB
from Screens.ChoiceBox import ChoiceBox
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from ServiceReference import ServiceReference
from Tools.ISO639 import LanguageCodes
from __init__ import _
import os

config.AC3LipSync = ConfigSubsection()
config.AC3LipSync.lowerBound = ConfigInteger(default = 0, limits = (0,10000))
config.AC3LipSync.upperBound = ConfigInteger(default = 405, limits = (0,10000))
config.AC3LipSync.arrowStepSize = ConfigInteger(default = 45, limits = (0,10000))
config.AC3LipSync.stepSize = ConfigInteger(default = 5, limits = (0,10000))

lFileDelay = {}
lFileDelay["Bitstream"] = "/proc/stb/audio/audio_delay_bitstream"
lFileDelay["PCM"] = "/proc/stb/audio/audio_delay_pcm"

def getCurrentDelay(sWhichAudio):
    sFileName = lFileDelay[sWhichAudio]
    if os.path.exists(sFileName) == True:
        delayfile = open(sFileName,"r")
        delay = 0
        delay = delayfile.readline()
        delayfile.close()
        iDelay = hex2dec(delay)/100
    else:
        iDelay = 0
    return int(iDelay)
        
class AC3LipSync(Screen):
    skin = """
        <screen position="60,370" size="600,150" title="AC3 Lip Sync" zPosition="1" backgroundColor="background" >
            <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/key-red.png" position="0,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
            <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/key-green.png" position="140,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
            <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/key-yellow.png" position="280,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
            <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/key-blue.png" position="420,0" zPosition="0" size="140,40" transparent="1" alphatest="on" />
            <widget name="key_red" position="0,0" zPosition="1" size="140,40"
                font="Regular;20" valign="center" halign="center" backgroundColor="#9f1313" transparent="1"
                shadowColor="#000000" shadowOffset="-1,-1" />
            <widget name="key_green" position="140,0" zPosition="1" size="140,40"
                font="Regular;20" valign="center" halign="center" backgroundColor="#1f771f" transparent="1"
                shadowColor="#000000" shadowOffset="-1,-1" />
            <widget name="key_yellow" position="280,0" zPosition="1" size="140,40"
                font="Regular;20" valign="center" halign="center" backgroundColor="#a08500" transparent="1"
                shadowColor="#000000" shadowOffset="-1,-1" />
            <widget name="key_blue" position="420,0" zPosition="1" size="140,40"
                font="Regular;20" valign="center" halign="center" backgroundColor="#18188b" transparent="1"
            shadowColor="#000000" shadowOffset="-1,-1" />
            <widget name="BitstreamDelayText" zPosition="2" position="30,50" size="180,21" font="Regular;21" foregroundColors="#ffffff,#ffa323" />
            <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/AC3LipSyncBarBG.png" zPosition="2" position="220,50" size="370,21" alphatest="on" transparent="1" />
            <widget name="BitstreamSlider" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/AC3LipSyncBar.png" zPosition="3" position="220,50" size="370,21" transparent="1" />
            <widget name="BitstreamSliderText" zPosition="4" position="220,50" size="370,21" font="Regular;18" halign="center" valign="center" transparent="1" />
            <widget name="PCMDelayText" zPosition="2" position="30,76" size="180,21" font="Regular;21" foregroundColors="#ffffff,#ffa323" />
            <ePixmap pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/AC3LipSyncBarBG.png" zPosition="2" position="220,76" size="370,21" alphatest="on" transparent="1" />
            <widget name="PCMSlider" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AC3LipSync/img/AC3LipSyncBar.png" zPosition="3" position="220,76" size="370,21" transparent="1" />
            <widget name="PCMSliderText" zPosition="4" position="220,76" size="370,21" font="Regular;18" halign="center" valign="center" transparent="1" />
            <widget name="ServiceInfo" zPosition="4" position="30,110" size="580,21" font="Regular;21" />
        </screen>"""
                
    def __init__(self, session, args = None):
        Screen.__init__(self, session)
        self.onShow.append(self.__onShow)

        self.getServiceReference()
        self.getAudioInformation()
        self.lamedb = AC3lamedb(session)
        self.lamedbName = str(self.lamedb.getServiceInfo(self.currentServiceRef).ChannelName)
        self.lamedbDelay = {}
        
        # What do the current keys work on
        self.whichAudio = "Bitstream"
        self.whichKeys = "Computed" # Computed = computed Values of keys, User = User set values of keys
        self.session = session
        self.lowerBound = int(config.AC3LipSync.lowerBound.getValue())
        self.upperBound = int(config.AC3LipSync.upperBound.getValue())
        self.arrowStepSize = int(config.AC3LipSync.arrowStepSize.getValue())
        self.stepSize = int(config.AC3LipSync.stepSize.getValue())
        self.BitstreamSliderBar = ProgressBar()
        self["BitstreamSlider"] = self.BitstreamSliderBar
        self["BitstreamSliderText"] = Label("0 " + _("ms"))
        self["BitstreamDelayText"] = MultiColorLabel(_("Bitstream delay:"))
        self.PCMSliderBar = ProgressBar()
        self["PCMSlider"] = self.PCMSliderBar
        self["PCMSliderText"] = Label("0 " + _("ms"))
        self["PCMDelayText"] = MultiColorLabel(_("PCM delay:"))
        self["ServiceInfo"] = Label()
        self.setChannelInfoText()
        
        self["key_red"] = Button(_("Cancel"))
        self["key_green"] = Button(_("Save"))
        self["key_yellow"] = Button(_(" "))
        self["key_blue"] = Button(_("lamedb..."))   
        
        self.savedValue = {}
        self.savedValue["Bitstream"] = 0
        self.savedValue["PCM"] = 0
        self["actions"] = NumberActionMap(["WizardActions", "NumberActions", "ColorActions"] , 
        {
            "ok": self.ok,
            "back": self.cancel,
            "left": self.keyLeft,
            "right": self.keyRight,
            "up": self.keyUp,
            "down": self.keyDown,
            "red": self.cancel,
            "green": self.ok,
            "blue": self.saveToLamedb,
            "1": self.keyNumberGlobal,
            "2": self.keyNumberGlobal,
            "3": self.keyNumberGlobal,
            "4": self.keyNumberGlobal,
            "5": self.keyNumberGlobal,
            "6": self.keyNumberGlobal,
            "7": self.keyNumberGlobal,
            "8": self.keyNumberGlobal,
            "9": self.keyNumberGlobal,
            "0": self.keyNumberGlobal
        }, -1)
            
    def __onShow(self):
        for sAudio in lFileDelay.keys():
            iDelay = getCurrentDelay(sAudio)
            self[sAudio + "Slider"].setRange([(self.lowerBound), (self.upperBound)])
            self[sAudio + "Slider"].setValue(iDelay)
            self[sAudio + "SliderText"].setText(str(iDelay)+" " + _("ms"))
            self.savedValue[sAudio] = iDelay
            self["BitstreamDelayText"].setForegroundColorNum(1)
            self["PCMDelayText"].setForegroundColorNum(0)
    
    def setChannelInfoText(self):
        self.lamedbDelay["Bitstream"] = self.lamedb.getNamedServiceParameter(self.currentServiceRef,"DelayBitstream",True)
        self.lamedbDelay["PCM"] = self.lamedb.getNamedServiceParameter(self.currentServiceRef,"DelayPCM",True)
        self["ServiceInfo"].setText(_("Channel: ") + str(self.lamedbName + "   " + _("Bitstream Delay: ") + str(self.lamedbDelay["Bitstream"])+ "   " + _("PCM Delay: ") + str(self.lamedbDelay["PCM"])))
 
    def getServiceReference(self):
        currentServiceRef = self.session.nav.getCurrentlyPlayingServiceReference()
        if currentServiceRef is not None:
            sCurrentServiceRef = currentServiceRef.toString()
            vCurrentServiceRef = sCurrentServiceRef.split(":")
            self.currentServiceRef = vCurrentServiceRef[3].lower()
        else:
            self.currentServiceRef = "N/A"

    
    def getAudioInformation(self):
        self.service = service = self.session.nav.getCurrentService()
        self.audioTracks = audio = service and service.audioTracks()
        n = audio and audio.getNumberOfTracks() or 0
        tlist = []
        self.selectedAudioIndex = self.audioTracks.getCurrentTrack()
        if n > 0:
            for x in range(n):
                i = audio.getTrackInfo(x)
                language = i.getLanguage()
                description = i.getDescription()

                if LanguageCodes.has_key(language):
                    language = LanguageCodes[language][0]

                if len(description):
                    description += " (" + language + ")"
                else:
                    description = language

                tlist.append((description, x))
                if x == self.selectedAudioIndex:
                    self.selectedAudioInfo = (description, x)
            tlist.sort(key=lambda x: x[0])

            self.audioTrackList = tlist
            
    def keyUp(self):
        self.whichAudio = "Bitstream"
        self["BitstreamDelayText"].setForegroundColorNum(1)
        self["PCMDelayText"].setForegroundColorNum(0)

    def keyDown(self):
        self.whichAudio = "PCM"
        self["BitstreamDelayText"].setForegroundColorNum(0)
        self["PCMDelayText"].setForegroundColorNum(1)
                    
    def keyLeft(self):
        sAudio = self.whichAudio
        iSliderValue = int(self[sAudio+"Slider"].getValue())
        iSliderValue -= self.arrowStepSize
        if iSliderValue < self.lowerBound:
            iSliderValue = 0
        self.setDelay(iSliderValue)
        
    def keyRight(self):
        sAudio = self.whichAudio
        iSliderValue = int(self[sAudio+"Slider"].getValue())
        if iSliderValue == 0:
            iSliderValue = self.lowerBound
        iSliderValue += self.arrowStepSize
        if iSliderValue > self.upperBound:
            iSliderValue = self.upperBound
        self.setDelay(iSliderValue)

    def keyNumberGlobal(self, number):
        iNumber = int(number)
        if iNumber == 0:
            iSliderValue = 0
        else:
            if self.whichKeys == "Computed":
                iSliderValue = self.lowerBound + self.stepSize*iNumber
            else:
                iSliderValue = int(config.AC3LipSync.steps[iNumber].getValue())
        self.setDelay(iSliderValue)
    
    def switchKeyValues(self):
        if self.whichKeys == "Computed":
            self.whichKeys = "User"
            self["key_blue"].setText(_("Use auto delays"))
        else:
            self.whichKeys = "Computed"
            self["key_blue"].setText(_("Use user delays"))
    
    def saveDelayToKey(self):
        AC3SetCustomValue(self.session,self.whichAudio)
   
    def saveToLamedb(self):
        keyList = []
        keyList.append((_("Save Bitstream delay"),"1"))
        keyList.append((_("Save PCM delay"),"2"))
        keyList.append((_("Save both delays"),"3"))
        keyList.append((_("Backup lamedb"),"4"))
        keyList.append((_("Restore lamedb"),"5"))
        keyList.append((_("Remove lamedb backup"),"6"))
        keyList.append((_("Reload lamedb"),"7"))

        self.session.openWithCallback(self.DoSaveLamedb,ChoiceBox,_("What you want to do with your lamedb"),keyList)
    
    def DoSaveLamedb(self, answer):
        sNewLine = ""
        sResponse = ""
        bOk = True
        iType = MessageBox.TYPE_INFO
        if answer is not None:
            if answer[1] in ("1","2","3"):
                if answer[1] in ("1","3"):
                    iCurDelay = self["BitstreamSlider"].getValue()
                    if iCurDelay == 0:
                        iNewDelay = 0
                    else:
                        iNewDelay = iCurDelay + int( self.lamedb.getNamedServiceParameter(self.currentServiceRef,"DelayBitstream",True) )
                    self.lamedb.setNamedServiceParameter(self.currentServiceRef,"DelayBitstream",iNewDelay,True)
                    self.whichAudio = "Bitstream"
                    self.setDelay(0)
                    sResponse = _("Saved Bitstream value: ") + str(self.lamedb.getNamedServiceParameter(self.currentServiceRef,"DelayBitstream",True))
                    sNewLine = "\n"
                if answer[1] in ("2","3"):
                    iCurDelay = self["PCMSlider"].getValue()
                    if iCurDelay == 0:
                        iNewDelay = 0
                    else:
                        iNewDelay = iCurDelay + int( self.lamedb.getNamedServiceParameter(self.currentServiceRef,"DelayPCM",True) )
                    self.lamedb.setNamedServiceParameter(self.currentServiceRef,"DelayPCM",iNewDelay,True)
                    self.whichAudio = "PCM"
                    self.setDelay(0)
                    sResponse = sResponse + sNewLine + _("Saved PCM value: ") + str(self.lamedb.getNamedServiceParameter(self.currentServiceRef,"DelayPCM",True))
                self.lamedb.saveServiceList()
            elif answer[1] == "4":
                self.lamedb.backupLameDb()
                bOk = False
            elif answer[1] == "5": 
                self.lamedb.restoreLameDb()
                bOk = False
            elif answer[1] == "6":
                self.lamedb.removeLameDbBackup()
                bOk = False
            elif answer[1] == "7":
                print "[AC3LipSync] reloading lamedb"        
                db = eDVBDB.getInstance()
                db.reloadServicelist()
                sResponse = _("Successfully reloaded lamedb")
            else:
                sResponse = _("Invalid selection")
                iType = MessageBox.TYPE_ERROR
            if bOk ==True:
                self.session.open(MessageBox, sResponse , iType)
        self.setChannelInfoText()
               
    def setDelay(self, iDelay):
        sAudio = self.whichAudio
        self[sAudio+"Slider"].setValue(iDelay)
        self[sAudio+"SliderText"].setText(str(iDelay)+" ms")
        delay = dec2hex(iDelay*100)
        sFileName = lFileDelay[sAudio]
        if os.path.exists(sFileName) == True:
            delayfile = open(lFileDelay[sAudio],"w")
            delayfile.write("%s\0" % delay)
            delayfile.close()

            if self.selectedAudioIndex == 0:
                self.audioTracks.selectTrack(1)
            else:
                self.audioTracks.selectTrack(0)
            self.audioTracks.selectTrack(self.selectedAudioIndex)
            
    def ok(self):
        self.close()
        
    def cancel(self):
        for sAudio in lFileDelay.keys():
            iSliderValue = int(self[sAudio + "Slider"].getValue())
            if iSliderValue <> self.savedValue[sAudio]:
                self.whichAudio = sAudio
                self.setDelay(self.savedValue[sAudio])
        self.close()

class AC3SetCustomValue(Screen):

    def __init__(self, session, sWhichAudio):
        self.session = session
        self.whichAudio = sWhichAudio
        self.session.openWithCallback(self.DoSetCustomValue,ChoiceBox,_("Select the key you want to set"),self.getKeyList())

    def getKeyList(self):
        keyList = []
        for i in range(1, 10):
            keyList.append((_("Key")+" "+str(i),str(i)))
        return keyList

    def DoSetCustomValue(self,answer):
        if answer is None:
            self.session.open(MessageBox,_("Setting key canceled"), MessageBox.TYPE_INFO)    
        else:
            iCurDelay = getCurrentDelay(self.whichAudio)
            config.AC3LipSync.steps[int(answer[1])].delay.setValue(iCurDelay)
            config.AC3LipSync.steps.save()
            self.session.open(MessageBox,answer[0] + _("successfully set to ") + str(iCurDelay) + _("ms"), MessageBox.TYPE_INFO)